@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * Popular encoders and decoders
- *                                                      Copyright (C) 2000-2004 Mikio Hirabayashi
+ *                                                      Copyright (C) 2000-2005 Mikio Hirabayashi
  * This file is part of QDBM, Quick Database Manager.
  * QDBM is free software; you can redistribute it and/or modify it under the terms of the GNU
  * Lesser General Public License as published by the Free Software Foundation; either version
@@ -18,6 +18,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <time.h>
 
 #undef TRUE
 #define TRUE           1                 /* boolean true */
@@ -50,6 +51,7 @@ int runcsv(int argc, char **argv);
 int runxml(int argc, char **argv);
 int runzlib(int argc, char **argv);
 int runiconv(int argc, char **argv);
+int rundate(int argc, char **argv);
 void shouucsmap(void);
 
 
@@ -76,6 +78,8 @@ int main(int argc, char **argv){
     rv = runzlib(argc, argv);
   } else if(!strcmp(argv[1], "iconv")){
     rv = runiconv(argc, argv);
+  } else if(!strcmp(argv[1], "date")){
+    rv = rundate(argc, argv);
   } else {
     usage();
   }
@@ -98,7 +102,7 @@ void usage(void){
   fprintf(stderr, "  %s csv [-d] [-t] [-l] [-e expr] [-html] [file]\n", progname);
   fprintf(stderr, "  %s xml [-d] [-p] [-l] [-e expr] [-tsv] [file]\n", progname);
   if((tmp = cbdeflate("", 0, &tsiz)) != NULL){
-    fprintf(stderr, "  %s zlib [-d] [file]\n", progname);
+    fprintf(stderr, "  %s zlib [-d] [-crc] [file]\n", progname);
     free(tmp);
   }
   if((tmp = cbiconv("", 0, "US-ASCII", "US-ASCII", NULL, NULL)) != NULL){
@@ -106,6 +110,7 @@ void usage(void){
             progname);
     free(tmp);
   }
+  fprintf(stderr, "  %s date [-wf] [-rf] [-utc] [str]\n", progname);
   fprintf(stderr, "\n");
   exit(1);
 }
@@ -340,8 +345,8 @@ int runquote(int argc, char **argv){
 int runmime(int argc, char **argv){
   CBMAP *attrs;
   CBLIST *parts;
-  int i, size, dec, line, qp, dc, hd, bd, pnum;
-  const char *key;
+  int i, size, dec, line, qp, dc, hd, bd, pnum, rsiz, bsiz;
+  const char *key, *body;
   char *code, *expr, *file, *buf, *res, renc[64];
   dec = FALSE;
   hd = FALSE;
@@ -399,12 +404,12 @@ int runmime(int argc, char **argv){
   }
   if(hd || bd || pnum > 0){
     attrs = cbmapopen();
-    res = cbmimebreak(buf, attrs);
+    res = cbmimebreak(buf, size, attrs, &rsiz);
     if(pnum > 0){
       parts = NULL;
       if(!(key = cbmapget(attrs, "TYPE", -1, NULL)) || !cbstrfwimatch(key, "multipart/") ||
-         !(key = cbmapget(attrs, "BOUNDARY", -1, NULL)) || !(parts = cbmimeparts(res, key)) ||
-         cblistnum(parts) < pnum){
+         !(key = cbmapget(attrs, "BOUNDARY", -1, NULL)) ||
+         !(parts = cbmimeparts(res, rsiz, key)) || cblistnum(parts) < pnum){
         fprintf(stderr, "%s: not multipart or no such part\n", progname);
         if(parts) cblistclose(parts);
         free(res);
@@ -412,7 +417,10 @@ int runmime(int argc, char **argv){
         free(buf);
         return 1;
       }
-      printf("%s", cblistval(parts, pnum - 1, NULL));
+      body = cblistval(parts, pnum - 1, &bsiz);
+      for(i = 0; i < bsiz; i++){
+        putchar(body[i]);
+      }
       cblistclose(parts);
     } else if(hd){
       cbmapiterinit(attrs);
@@ -420,7 +428,9 @@ int runmime(int argc, char **argv){
         printf("%s\t%s\n", key, cbmapget(attrs, key, -1, NULL));
       }
     } else {
-      printf("%s", res);
+      for(i = 0; i < rsiz; i++){
+        putchar(res[i]);
+      }
     }
     free(res);
     cbmapclose(attrs);
@@ -493,30 +503,28 @@ int runcsv(int argc, char **argv){
       cells = cbcsvcells(row);
       for(j = 0; j < cblistnum(cells); j++){
         cell = cblistval(cells, j, NULL);
-        res = cbcsvunescape(cell);
         if(html){
           printf("<td>");
-          for(k = 0; res[k] != '\0'; k++){
-            if(res[k] == '\r' || res[k] == '\n'){
+          for(k = 0; cell[k] != '\0'; k++){
+            if(cell[k] == '\r' || cell[k] == '\n'){
               printf("<br>");
-              if(res[k] == '\r' && res[k] == '\n') k++;
+              if(cell[k] == '\r' && cell[k] == '\n') k++;
             } else {
-              switch(res[k]){
+              switch(cell[k]){
               case '&': printf("&amp;"); break;
               case '<': printf("&lt;"); break;
               case '>': printf("&gt;"); break;
-              default: putchar(res[k]); break;
+              default: putchar(cell[k]); break;
               }
             }
           }
           printf("</td>");
         } else {
           if(j > 0) putchar('\t');
-          for(k = 0; res[k] != '\0'; k++){
-            if(((unsigned char *)res)[k] >= 0x20) putchar(res[k]);
+          for(k = 0; cell[k] != '\0'; k++){
+            if(((unsigned char *)cell)[k] >= 0x20) putchar(cell[k]);
           }
         }
-        free(res);
       }
       cblistclose(cells);
       if(html) printf("</tr>");
@@ -622,16 +630,9 @@ int runxml(int argc, char **argv){
           cbmapiterinit(attrs);
           for(j = 0; (attr = cbmapiternext(attrs, NULL)) != NULL; j++){
             if(j < 1){
-              res = cbxmlunescape(cbmapget(attrs, attr, -1, NULL));
-              printf("%s", res);
-              free(res);
+              printf("%s", cbmapget(attrs, attr, -1, NULL));
             } else {
-              res = cbxmlunescape(attr);
-              printf("\t%s", res);
-              free(res);
-              res = cbxmlunescape(cbmapget(attrs, attr, -1, NULL));
-              printf("\t%s", res);
-              free(res);
+              printf("\t%s\t%s", attr, cbmapget(attrs, attr, -1, NULL));
             }
           }
           cbmapclose(attrs);
@@ -673,14 +674,18 @@ int runxml(int argc, char **argv){
 
 /* parse arguments of zlib command */
 int runzlib(int argc, char **argv){
-  int i, bsiz, rsiz, dec;
+  unsigned int sum;
+  int i, bsiz, rsiz, dec, crc;
   char *file, *buf, *res;
   dec = FALSE;
+  crc = FALSE;
   file = NULL;
   for(i = 2; i < argc; i++){
     if(!file && argv[i][0] == '-'){
       if(!strcmp(argv[i], "-d")){
         dec = TRUE;
+      } else if(!strcmp(argv[i], "-crc")){
+        crc = TRUE;
       } else {
         usage();
       }
@@ -699,7 +704,14 @@ int runzlib(int argc, char **argv){
   } else {
     buf = readstdin(&bsiz);
   }
-  if(dec){
+  if(crc){
+    sum = cbgetcrc(buf, bsiz);
+    for(i = 0; i < sizeof(int); i++){
+      printf("%02x", sum / 0x1000000);
+      sum <<= 8;
+    }
+    putchar('\n');
+  } else if(dec){
     if(!(res = cbinflate(buf, bsiz, &rsiz))){
       fprintf(stderr, "%s: inflate failed\n", progname);
       free(buf);
@@ -836,6 +848,59 @@ int runiconv(int argc, char **argv){
 }
 
 
+/* parse arguments of date command */
+int rundate(int argc, char **argv){
+  int i, wb, rb, utc, jl;
+  char *date, *res;
+  time_t t;
+  wb = FALSE;
+  rb = FALSE;
+  utc = FALSE;
+  date = NULL;
+  for(i = 2; i < argc; i++){
+    if(!date && argv[i][0] == '-'){
+      if(!strcmp(argv[i], "-wf")){
+        wb = TRUE;
+      } else if(!strcmp(argv[i], "-rf")){
+        rb = TRUE;
+      } else if(!strcmp(argv[i], "-utc")){
+        utc = TRUE;
+      } else {
+        usage();
+      }
+    } else if(!date){
+      date = argv[i];
+    } else {
+      usage();
+    }
+  }
+  jl = utc ? 0 : cbjetlag();
+  if(date){
+    t = cbstrmktime(date);
+  } else {
+    t = time(NULL);
+  }
+  if(wb){
+    res = cbdatestrwww(t, jl);
+  } else if(rb){
+    res = cbdatestrhttp(t, jl);
+  } else {
+    res = cbsprintf("%d", (int)t);
+  }
+  if(t >= 0){
+    printf("%s\n", res);
+  } else {
+    if(date){
+      fprintf(stderr, "%s: %s: invalid date format\n", progname, date);
+    } else {
+      fprintf(stderr, "%s: invalid time setting\n", progname);
+    }
+  }
+  free(res);
+  return 0;
+}
+
+
 /* show mapping of UCS-2 and exit. */
 void shouucsmap(void){
   unsigned char buf[2], *tmp;
