@@ -220,7 +220,7 @@ int runmisc(int argc, char **argv){
 
 
 /* print formatted string and flush the buffer */
-int prinfflush(const char *format, ...){
+int printfflush(const char *format, ...){
   va_list ap;
   int rv;
   va_start(ap, format);
@@ -253,7 +253,7 @@ int dosort(int rnum, int disp){
   char **ivector1, **ivector2, **ivector3, **ivector4, **ivector5;
   char buf[RECBUFSIZ];
   int i, len, err;
-  if(!disp) prinfflush("<Sorting Test>\n  rnum=%d  disp=%d\n\n", rnum, disp);
+  if(!disp) printfflush("<Sorting Test>\n  rnum=%d  disp=%d\n\n", rnum, disp);
   ivector1 = cbmalloc(rnum * sizeof(ivector1[0]));
   ivector2 = cbmalloc(rnum * sizeof(ivector2[0]));
   ivector3 = cbmalloc(rnum * sizeof(ivector3[0]));
@@ -268,20 +268,20 @@ int dosort(int rnum, int disp){
     ivector4[i] = cbmemdup(buf, len);
     ivector5[i] = cbmemdup(buf, len);
   }
-  if(!disp) prinfflush("Sorting with insert sort ... ");
+  if(!disp) printfflush("Sorting with insert sort ... ");
   cbisort(ivector1, rnum, sizeof(ivector1[0]), strpcmp);
-  if(!disp) prinfflush("ok\n");
-  if(!disp) prinfflush("Sorting with shell sort ... ");
+  if(!disp) printfflush("ok\n");
+  if(!disp) printfflush("Sorting with shell sort ... ");
   cbssort(ivector2, rnum, sizeof(ivector2[0]), strpcmp);
-  if(!disp) prinfflush("ok\n");
-  if(!disp) prinfflush("Sorting with heap sort ... ");
+  if(!disp) printfflush("ok\n");
+  if(!disp) printfflush("Sorting with heap sort ... ");
   cbhsort(ivector3, rnum, sizeof(ivector3[0]), strpcmp);
-  if(!disp) prinfflush("ok\n");
-  if(!disp) prinfflush("Sorting with quick sort ... ");
+  if(!disp) printfflush("ok\n");
+  if(!disp) printfflush("Sorting with quick sort ... ");
   cbqsort(ivector4, rnum, sizeof(ivector4[0]), strpcmp);
-  if(!disp) prinfflush("ok\n");
+  if(!disp) printfflush("ok\n");
   for(i = 0; i < rnum; i++){
-    if(disp) prinfflush("%s\t%s\t%s\t%s\t[%s]\n",
+    if(disp) printfflush("%s\t%s\t%s\t%s\t[%s]\n",
                         ivector1[i], ivector2[i], ivector3[i], ivector4[i], ivector5[i]);
     if(strcmp(ivector1[i], ivector2[i])) err = TRUE;
     if(strcmp(ivector1[i], ivector3[i])) err = TRUE;
@@ -298,7 +298,7 @@ int dosort(int rnum, int disp){
   free(ivector4);
   free(ivector5);
   if(err) fprintf(stderr, "%s: sorting failed\n", progname);
-  if(!disp && !err) prinfflush("all ok\n\n");
+  if(!disp && !err) printfflush("all ok\n\n");
   return err ? 1 : 0;
 }
 
@@ -309,7 +309,7 @@ int dolist(int rnum, int disp){
   int i, err, len;
   char buf[RECBUFSIZ];
   const char *val;
-  if(!disp) prinfflush("<List Writing Test>\n  rnum=%d  disp=%d\n\n", rnum, disp);
+  if(!disp) printfflush("<List Writing Test>\n  rnum=%d  disp=%d\n\n", rnum, disp);
   list = cblistopen();
   err = FALSE;
   for(i = 1; i <= rnum; i++){
@@ -319,14 +319,14 @@ int dolist(int rnum, int disp){
       putchar('.');
       fflush(stdout);
       if(i == rnum || i % (rnum / 10) == 0){
-        prinfflush(" (%08d)\n", i);
+        printfflush(" (%08d)\n", i);
       }
     }
   }
   if(disp){
     for(i = 0; i < cblistnum(list); i++){
       if((val = cblistval(list, i, NULL)) != NULL){
-        prinfflush("%s\n", val);
+        printfflush("%s\n", val);
       } else {
         fprintf(stderr, "%s: val error\n", progname);
         err = TRUE;
@@ -335,7 +335,7 @@ int dolist(int rnum, int disp){
     }
   }
   cblistclose(list);
-  if(!disp && !err) prinfflush("ok\n\n");
+  if(!disp && !err) printfflush("ok\n\n");
   return err ? 1 : 0;
 }
 
@@ -346,7 +346,7 @@ int domap(int rnum, int bnum, int disp){
   int i, err, len;
   char buf[RECBUFSIZ];
   const char *val;
-  if(!disp) prinfflush("<Map Writing Test>\n  rnum=%d  bnum=%d  disp=%d\n\n", rnum, bnum, disp);
+  if(!disp) printfflush("<Map Writing Test>\n  rnum=%d  bnum=%d  disp=%d\n\n", rnum, bnum, disp);
   map = bnum > 0 ? cbmapopenex(bnum) : cbmapopen();
   err = FALSE;
   for(i = 1; i <= rnum; i++){
@@ -360,7 +360,7 @@ int domap(int rnum, int bnum, int disp){
       putchar('.');
       fflush(stdout);
       if(i == rnum || i % (rnum / 10) == 0){
-        prinfflush(" (%08d)\n", i);
+        printfflush(" (%08d)\n", i);
       }
     }
   }
@@ -368,14 +368,14 @@ int domap(int rnum, int bnum, int disp){
     for(i = 1; i <= rnum; i++){
       len = sprintf(buf, "%08d", i);
       if((val = cbmapget(map, buf, len, NULL)) != NULL){
-        prinfflush("%s\n", val);
+        printfflush("%s\n", val);
       } else {
         fprintf(stderr, "%s: get error\n", progname);
       }
     }
   }
   cbmapclose(map);
-  if(!disp && !err) prinfflush("ok\n\n");
+  if(!disp && !err) printfflush("ok\n\n");
   return err ? 1 : 0;
 }
 
@@ -385,8 +385,8 @@ int dowicked(int rnum){
   CBLIST *list;
   CBMAP *map;
   int i, len;
-  char buf[RECBUFSIZ];
-  prinfflush("<Wicked Writing Test>\n  rnum=%d\n\n", rnum);
+  char buf[RECBUFSIZ], *tmp;
+  printfflush("<Wicked Writing Test>\n  rnum=%d\n\n", rnum);
   list = cblistopen();
   for(i = 1; i <= rnum; i++){
     len = sprintf(buf, "%d", myrand() % rnum + 1);
@@ -415,12 +415,17 @@ int dowicked(int rnum){
       cblistover(list, myrand() % (cblistnum(list) + 1), buf, len);
       putchar('V');
       break;
+    case 6:
+      tmp = cbmemdup(buf, len);
+      cblistpushbuf(list, tmp, len);
+      putchar('B');
+      break;
     default:
       cblistpush(list, buf, len);
       putchar('P');
       break;
     }
-    if(i % 50 == 0) prinfflush(" (%08d)\n", i);
+    if(i % 50 == 0) printfflush(" (%08d)\n", i);
   }
   cblistclose(list);
   map = cbmapopen();
@@ -447,15 +452,20 @@ int dowicked(int rnum){
       cbmapmove(map, buf, len, myrand() % 2);
       putchar('M');
       break;
+    case 5:
+      tmp = cbmemdup(buf, len);
+      cbmapputvbuf(map, buf, len, tmp, len);
+      putchar('B');
+      break;
     default:
       cbmapput(map, buf, len, buf, len, TRUE);
       putchar('O');
       break;
     }
-    if(i % 50 == 0) prinfflush(" (%08d)\n", i);
+    if(i % 50 == 0) printfflush(" (%08d)\n", i);
   }
   cbmapclose(map);
-  prinfflush("ok\n\n");
+  printfflush("ok\n\n");
   return 0;
 }
 
@@ -464,12 +474,12 @@ int dowicked(int rnum){
 int domisc(void){
   CBLIST *olist, *nlist, *elems, *glist;
   CBMAP *omap, *nmap, *pairs, *gmap;
-  int i, j, ssiz, osiz, jl;
+  int i, j, ssiz, osiz, tsiz, jl;
   char kbuf[RECBUFSIZ], vbuf[RECBUFSIZ], *sbuf, spbuf[1024], *tmp, *orig, renc[64];
   const char *op, *np;
   time_t t;
-  prinfflush("<Miscellaneous Test>\n\n");
-  prinfflush("Checking serialization of list ... ");
+  printfflush("<Miscellaneous Test>\n\n");
+  printfflush("Checking serialization of list ... ");
   olist = cblistopen();
   for(i = 1; i <= 1000; i++){
     sprintf(vbuf, "%d", i);
@@ -490,15 +500,18 @@ int domisc(void){
   }
   cblistclose(nlist);
   cblistclose(olist);
-  prinfflush("ok\n");
-  prinfflush("Checking serialization of map ... ");
+  printfflush("ok\n");
+  printfflush("Checking serialization of map ... ");
   omap = cbmapopen();
   for(i = 1; i <= 1000; i++){
     sprintf(kbuf, "%X", i);
     sprintf(vbuf, "[%d]", i);
+    cbmapput(omap, kbuf, -1, vbuf, -1, TRUE);
   }
   sbuf = cbmapdump(omap, &ssiz);
   nmap = cbmapload(sbuf, ssiz);
+  free(cbmaploadone(sbuf, ssiz, "1", 2, &tsiz));
+  free(cbmaploadone(sbuf, ssiz, "33", 2, &tsiz));
   free(sbuf);
   cbmapiterinit(omap);
   while((op = cbmapiternext(omap, NULL)) != NULL){
@@ -511,8 +524,8 @@ int domisc(void){
   }
   cbmapclose(nmap);
   cbmapclose(omap);
-  prinfflush("ok\n");
-  prinfflush("Checking string utilities ... ");
+  printfflush("ok\n");
+  printfflush("Checking string utilities ... ");
   sprintf(spbuf, "[%08d/%08o/%08u/%08x/%08X/%08.3e/%08.3E/%08.3f/%08.3g/%08.3G/%c/%s/%%]",
           123456, 123456, 123456, 123456, 123456,
           123456.789, 123456.789, 123456.789, 123456.789, 123456.789,
@@ -570,25 +583,30 @@ int domisc(void){
     fprintf(stderr, "%s: cbstrbwimatch is invalid\n", progname);
     return 1;
   }
-  tmp = cbmemdup(" \r\n[Quick Database Manager]\r\n ", -1);
-  if(cbstrtoupper(tmp) != tmp || strcmp(tmp, " \r\n[QUICK DATABASE MANAGER]\r\n ")){
+  tmp = cbmemdup(" \r\n[Quick   Database Manager]\r\n ", -1);
+  if(cbstrtoupper(tmp) != tmp || strcmp(tmp, " \r\n[QUICK   DATABASE MANAGER]\r\n ")){
     free(tmp);
     fprintf(stderr, "%s: cbstrtoupper is invalid\n", progname);
     return 1;
   }
-  if(cbstrtolower(tmp) != tmp || strcmp(tmp, " \r\n[quick database manager]\r\n ")){
+  if(cbstrtolower(tmp) != tmp || strcmp(tmp, " \r\n[quick   database manager]\r\n ")){
     free(tmp);
     fprintf(stderr, "%s: cbstrtolower is invalid\n", progname);
     return 1;
   }
-  if(cbstrtrim(tmp) != tmp || strcmp(tmp, "[quick database manager]")){
+  if(cbstrtrim(tmp) != tmp || strcmp(tmp, "[quick   database manager]")){
     free(tmp);
     fprintf(stderr, "%s: cbstrtrim is invalid\n", progname);
     return 1;
   }
+  if(cbstrsqzspc(tmp) != tmp || strcmp(tmp, "[quick database manager]")){
+    free(tmp);
+    fprintf(stderr, "%s: cbstrsqzspc is invalid\n", progname);
+    return 1;
+  }
   free(tmp);
-  prinfflush("ok\n");
-  prinfflush("Checking encoding utilities ... ");
+  printfflush("ok\n");
+  printfflush("Checking encoding utilities ... ");
   strcpy(spbuf, "My name is \xca\xbf\xce\xd3\xb4\xb4\xcd\xba.\n\n<Love & Peace!>\n");
   tmp = cbbaseencode(spbuf, -1);
   orig = cbbasedecode(tmp, &osiz);
@@ -644,12 +662,12 @@ int domisc(void){
   }
   free(orig);
   free(tmp);
-  prinfflush("ok\n");
-  prinfflush("Checking date utilities ... ");
+  printfflush("ok\n");
+  printfflush("Checking date utilities ... ");
   for(i = 0; i < 200; i++){
     jl = (myrand() % 23) * 1800;
     if(myrand() % 2 == 0) jl *= -1;
-    t = myrand() % (INT_MAX - 3600 * 24) + 3600 * 24;
+    t = myrand() % (INT_MAX - 3600 * 24 * 365 * 6) + 3600 * 24 * 365 * 5;
     tmp = cbdatestrwww(t, jl);
     t = cbstrmktime(tmp);
     orig = cbdatestrwww(t, jl);
@@ -667,14 +685,14 @@ int domisc(void){
     if(strcmp(orig, tmp)){
       free(orig);
       free(tmp);
-      fprintf(stderr, "%s: RFC 1123 formatter is invalid\n", progname);
+      fprintf(stderr, "%s: RFC 822 date formatter is invalid\n", progname);
       return 1;
     }
     free(orig);
     free(tmp);
   }
-  prinfflush("ok\n");
-  prinfflush("Checking the global garbage collector ... ");
+  printfflush("ok\n");
+  printfflush("Checking the global garbage collector ... ");
   for(i = 0; i < 512; i++){
     glist = cblistopen();
     cbglobalgc(glist, (void (*)(void *))cblistclose);
@@ -690,8 +708,8 @@ int domisc(void){
     }
     if(myrand() % 64 == 0) cbggcsweep();
   }
-  prinfflush("ok\n");
-  prinfflush("all ok\n\n");
+  printfflush("ok\n");
+  printfflush("all ok\n\n");
   return 0;
 }
 
