@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * The basic API of QDBM
- *                                                      Copyright (C) 2000-2004 Mikio Hirabayashi
+ *                                                      Copyright (C) 2000-2005 Mikio Hirabayashi
  * This file is part of QDBM, Quick Database Manager.
  * QDBM is free software; you can redistribute it and/or modify it under the terms of the GNU
  * Lesser General Public License as published by the Free Software Foundation; either version
@@ -35,6 +35,7 @@ typedef struct {                         /* type of structure for a database han
   char *name;                            /* name of the database file */
   int wmode;                             /* whether writable or not */
   int inode;                             /* inode of the database file */
+  int mtime;                             /* last modified time of the database */
   int fd;                                /* file descriptor of the database file */
   int fsiz;                              /* size of the database file */
   char *map;                             /* pointer to the mapped memory */
@@ -78,7 +79,9 @@ enum {                                   /* enumeration for open modes */
   DP_OWRITER = 1 << 1,                   /* open as a writer */
   DP_OCREAT = 1 << 2,                    /* a writer creating */
   DP_OTRUNC = 1 << 3,                    /* a writer truncating */
-  DP_ONOLCK = 1 << 4                     /* open without locking */
+  DP_ONOLCK = 1 << 4,                    /* open without locking */
+  DP_OLCKNB = 1 << 5,                    /* lock without blocking */
+  DP_OSPARSE = 1 << 6                    /* create as a sparse file */
 };
 
 enum {                                   /* enumeration for write modes */
@@ -109,7 +112,9 @@ const char *dperrmsg(int ecode);
    If the mode is `DP_OWRITER', the following may be added by bitwise or: `DP_OCREAT', which
    means it creates a new database if not exist, `DP_OTRUNC', which means it creates a new
    database regardless if one exists.  Both of `DP_OREADER' and `DP_OWRITER' can be added to by
-   bitwise or: `DP_ONOLCK', which means it opens a database file without file locking.
+   bitwise or: `DP_ONOLCK', which means it opens a database file without file locking, or
+   `DP_OLCKNB', which means locking is performed without blocking.  `DP_OCREAT' can be added to
+   by bitwise or: `DP_OSPARSE', which means it creates a database file as a sparse file.
    `bnum' specifies the number of elements of the bucket array.  If it is not more than 0,
    the default value is specified.  The size of a bucket array is determined on creating,
    and can not be changed except for by optimization of the database.  Suggested size of a
@@ -312,6 +317,12 @@ int dpfatalerror(DEPOT *depot);
 int dpinode(DEPOT *depot);
 
 
+/* Get the last modified time of a database.
+   `depot' specifies a database handle.
+   The return value is the last modified time of the database. */
+int dpmtime(DEPOT *depot);
+
+
 /* Get the file descriptor of a database file.
    `depot' specifies a database handle.
    The return value is the file descriptor of the database file.
@@ -333,6 +344,21 @@ int dpremove(const char *name);
 int dprepair(const char *name);
 
 
+/* Dump all records as endian independent data.
+   `depot' specifies a database handle.
+   `name' specifies the name of an output file.
+   If successful, the return value is true, else, it is false. */
+int dpexportdb(DEPOT *depot, const char *name);
+
+
+/* Load all records from endian independent data.
+   `depot' specifies a database handle connected as a writer.  The database of the handle must
+   be empty.
+   `name' specifies the name of an input file.
+   If successful, the return value is true, else, it is false. */
+int dpimportdb(DEPOT *depot, const char *name);
+
+
 /* Hash function used inside Depot.
    `kbuf' specifies the pointer to the region of a key.
    `ksiz' specifies the size of the region of the key.  If it is negative, the size is assigned
@@ -361,10 +387,14 @@ int dpprimenum(int num);
 
 
 /*************************************************************************************************
- * Features for Experts
+ * features for experts
  *************************************************************************************************/
 
 
+#define _QDBM_VERSION  "1.8.30"
+#define _QDBM_LIBVER   1102
+
+
 /* Name of the operating system. */
 extern char *dpsysname;
 
@@ -373,6 +403,10 @@ extern char *dpsysname;
 extern int dpdbgfd;
 
 
+/* Whether this build is reentrant. */
+extern const int dpisreentrant;
+
+
 /* Set the last happened error code.
    `ecode' specifies the error code.
    `line' specifies the number of the line where the error happened. */
