@@ -81,8 +81,10 @@ public class Villa implements ADBM {
   public static final int OTRUNC = 1 << 3;
   /** open mode: open without locking */
   public static final int ONOLCK = 1 << 4;
+  /** open mode: lock without blocking */
+  public static final int OLCKNB = 1 << 5;
   /** open mode: compress leaves */
-  public static final int OZCOMP = 1 << 5;
+  public static final int OZCOMP = 1 << 6;
   //----------------------------------------------------------------
   // comparing modes
   //----------------------------------------------------------------
@@ -170,7 +172,7 @@ public class Villa implements ADBM {
    * which means it creates a new database regardless if one exists, `Villa.OZCOMP', which means
    * leaves in the database are compressed.  Both of `Villa.OREADER' and `Villa.OWRITER' can be
    * added to by bitwise or: `Villa.ONOLCK', which means it opens a database file without file
-   * locking.
+   * locking, or `Villa.OLCKNB', which means locking is performed without blocking.
    * @param cmode the comparing function: `Villa.CMPLEX' comparing keys in lexical order,
    * `Villa.CMPNUM' comparing keys as numbers of big endian, `Villa.CMPDEC' comparing keys as
    * decimal strings, `Villa.CMPOBJ' comparing keys as serialized objects implementing
