@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * Implementation of Odeum
- *                                                      Copyright (C) 2000-2004 Mikio Hirabayashi
+ *                                                      Copyright (C) 2000-2005 Mikio Hirabayashi
  * This file is part of QDBM, Quick Database Manager.
  * QDBM is free software; you can redistribute it and/or modify it under the terms of the GNU
  * Lesser General Public License as published by the Free Software Foundation; either version
@@ -21,24 +21,31 @@
 #define OD_DIRMODE     00755             /* permission of a creating directory */
 #define OD_PATHBUFSIZ  1024              /* size of a path buffer */
 #define OD_NUMBUFSIZ   32                /* size of a buffer for a number */
+#define OD_MAPPBNUM    127               /* bucket size of a petit map handle */
 #define OD_DOCSNAME    "docs"            /* name of the database for documents */
 #define OD_INDEXNAME   "index"           /* name of the database for inverted index */
 #define OD_RDOCSNAME   "rdocs"           /* name of the database for reverse dictionary */
-#define OD_DOCSBNUM    1021              /* initial bucket number of document database */
-#define OD_DOCSDNUM    11                /* division number of document database */
+#define OD_DOCSBNUM    2039              /* initial bucket number of document database */
+#define OD_DOCSDNUM    17                /* division number of document database */
 #define OD_DOCSALIGN   -4                /* alignment of document database */
-#define OD_INDEXBNUM   16381             /* initial bucket number of inverted index */
+#define OD_INDEXBNUM   32749             /* initial bucket number of inverted index */
 #define OD_INDEXDNUM   7                 /* division number of inverted index */
 #define OD_INDEXALIGN  -2                /* alignment of inverted index */
 #define OD_RDOCSLRM    81                /* records in a leaf node of reverse dictionary */
 #define OD_RDOCSNIM    192               /* records in a non-leaf node of reverse dictionary */
 #define OD_RDOCSLCN    128               /* number of leaf cache of reverse dictionary */
 #define OD_RDOCSNCN    32                /* number of non-leaf cache of reverse dictionary */
+#define OD_CACHEBNUM   262139            /* number of buckets for dirty buffers */
+#define OD_CACHESIZ    8388608           /* max bytes to use memory for dirty buffers */
+#define OD_CFLIVERAT   0.8               /* ratio of usable cache region */
+#define OD_CFBEGSIZ    2048              /* beginning size of flushing frequent words */
+#define OD_CFENDSIZ    64                /* lower limit of flushing frequent words */
+#define OD_CFRFRAT     0.2               /* ratio of flushing rare words a time */
 #define OD_OTCBBUFSIZ  1024              /* size of a buffer for call back functions */
 #define OD_OTPERWORDS  10000             /* frequency of call back in merging index */
 #define OD_OTPERDOCS   1000              /* frequency of call back in merging docs */
 #define OD_MDBRATIO    2.5               /* ratio of bucket number and document number */
-#define OD_MIBRATIO    17.5              /* ratio of bucket number and word number */
+#define OD_MIBRATIO    1.5               /* ratio of bucket number and word number */
 #define OD_MIARATIO    0.75              /* ratio of alignment to the first words */
 #define OD_MIWUNIT     32                /* writing unit of merging inverted index */
 #define OD_DMAXEXPR    "dmax"            /* key of max number of the document ID */
@@ -52,8 +59,8 @@
 #define OD_KEYCRATIO   1.75              /* ratio of number to max of keyword candidates */
 #define OD_WOCCRPOINT  10000             /* points per occurence */
 #define OD_SPACECHARS  "\t\n\v\f\r "     /* space characters */
-#define OD_DELIMCHARS  "!\"#$%&'()*/<=>?@[\\]^`{|}~"  /* delimiter characters */
-#define OD_GLUECHARS   "+,-.:;"          /* glueing characters */
+#define OD_DELIMCHARS  "!\"#$%&'()*/<=>?[\\]^`{|}~"  /* delimiter characters */
+#define OD_GLUECHARS   "+,-.:;@"         /* glueing characters */
 #define OD_MAXWORDLEN  48                /* max length of a word */
 
 typedef struct {                         /* type of structure for word counting */
@@ -61,18 +68,40 @@ typedef struct {                         /* type of structure for word counting
   int num;                               /* frequency of the word */
 } ODWORD;
 
+enum {                                   /* enumeration for events binded to each character */
+  OD_EVWORD,                             /* word */
+  OD_EVSPACE,                            /* space */
+  OD_EVDELIM,                            /* delimiter */
+  OD_EVGLUE                              /* glue */
+};
+
 
 /* private global variables */
+int odindexbnum = OD_INDEXBNUM;
+int odindexdnum = OD_INDEXDNUM;
+int odcachebnum = OD_CACHEBNUM;
+int odcachesiz = OD_CACHESIZ;
 void (*odotcb)(const char *, ODEUM *, const char *) = NULL;
 
 
 /* private function prototypes */
-static ODEUM *odopendb(const char *name, int omode, int docsbnum, int indexbnum);
-static int odsortindex(ODEUM *odeum);
+static ODEUM *odopendb(const char *name, int omode, int docsbnum, int indexbnum,
+                       const char *fname);
+static int odcacheflush(ODEUM *odeum, const char *fname);
+static int odcacheflushfreq(ODEUM *odeum, const char *fname, int min);
+static int odcacheflushrare(ODEUM *odeum, const char *fname, double ratio);
+static int odsortindex(ODEUM *odeum, const char *fname);
 static int odsortcompare(const void *a, const void *b);
-static int odpurgeindex(ODEUM *odeum);
+static int odpurgeindex(ODEUM *odeum, const char *fname);
 static CBMAP *odpairsmap(const ODPAIR *pairs, int num);
 static int odwordcompare(const void *a, const void *b);
+static int odmatchoperator(ODEUM *odeum, CBLIST *tokens);
+static ODPAIR *odparsesubexpr(ODEUM *odeum, CBLIST *tokens, CBLIST *nwords, int *np,
+                              CBLIST *errors);
+static ODPAIR *odparseexpr(ODEUM *odeum, CBLIST *tokens, CBLIST *nwords, int *np,
+                           CBLIST *errors);
+static void odfixtokens(ODEUM *odeum, CBLIST *tokens);
+static void odcleannormalized(ODEUM *odeum, CBLIST *nwords);
 
 
 
@@ -84,7 +113,7 @@ static int odwordcompare(const void *a, const void *b);
 /* Get a database handle. */
 ODEUM *odopen(const char *name, int omode){
   assert(name);
-  return odopendb(name, omode, OD_DOCSBNUM, OD_INDEXBNUM);
+  return odopendb(name, omode, OD_DOCSBNUM, odindexbnum, "odopen");
 }
 
 
@@ -94,12 +123,18 @@ int odclose(ODEUM *odeum){
   int err;
   assert(odeum);
   err = FALSE;
+  if(odotcb) odotcb("odclose", odeum, "closing the connection");
   if(odeum->wmode){
+    if(odotcb) odotcb("odclose", odeum, "writing meta information");
     sprintf(numbuf, "%d", odeum->dmax);
     if(!vlput(odeum->rdocsdb, OD_DMAXEXPR, sizeof(OD_DMAXEXPR), numbuf, -1, VL_DOVER)) err = TRUE;
     sprintf(numbuf, "%d", odeum->dnum);
     if(!vlput(odeum->rdocsdb, OD_DNUMEXPR, sizeof(OD_DNUMEXPR), numbuf, -1, VL_DOVER)) err = TRUE;
-    if(!odsortindex(odeum)) err = TRUE;
+    if(!odcacheflushfreq(odeum, "odclose", OD_CFENDSIZ)) err = TRUE;
+    if(!odcacheflushrare(odeum, "odclose", OD_CFRFRAT)) err = TRUE;
+    if(!odcacheflush(odeum, "odclose")) err = TRUE;
+    if(!odsortindex(odeum, "odclose")) err = TRUE;
+    cbmapclose(odeum->cachemap);
     cbmapclose(odeum->sortmap);
   }
   if(!vlclose(odeum->rdocsdb)) err = TRUE;
@@ -151,33 +186,27 @@ int odput(ODEUM *odeum, ODDOC *doc, int wmax, int over){
   tmp = cbmapdump(doc->attrs, &tsiz);
   cbmapput(map, OD_ATTRSEXPR, sizeof(OD_ATTRSEXPR), tmp, tsiz, TRUE);
   free(tmp);
-  if(wmax >= 0 && wmax < cblistnum(doc->nwords)){
-    tlist = cblistopen();
-    for(i = 0; i < wmax; i++){
-      ctmp = cblistval(doc->nwords, i, &wsiz);
-      cblistpush(tlist, ctmp, wsiz);
-    }
-    tmp = cblistdump(tlist, &tsiz);
-    cbmapput(map, OD_NWORDSEXPR, sizeof(OD_NWORDSEXPR), tmp, tsiz, TRUE);
-    free(tmp);
-    cblistclose(tlist);
-    tlist = cblistopen();
-    for(i = 0; i < wmax; i++){
-      ctmp = cblistval(doc->awords, i, &wsiz);
-      cblistpush(tlist, ctmp, wsiz);
-    }
-    tmp = cblistdump(tlist, &tsiz);
-    cbmapput(map, OD_AWORDSEXPR, sizeof(OD_AWORDSEXPR), tmp, tsiz, TRUE);
-    free(tmp);
-    cblistclose(tlist);
-  } else {
-    tmp = cblistdump(doc->nwords, &tsiz);
-    cbmapput(map, OD_NWORDSEXPR, sizeof(OD_NWORDSEXPR), tmp, tsiz, TRUE);
-    free(tmp);
-    tmp = cblistdump(doc->awords, &tsiz);
-    cbmapput(map, OD_AWORDSEXPR, sizeof(OD_AWORDSEXPR), tmp, tsiz, TRUE);
-    free(tmp);
+  if(wmax < 0 || wmax > cblistnum(doc->nwords)) wmax = cblistnum(doc->nwords);
+  tlist = cblistopen();
+  for(i = 0; i < wmax; i++){
+    ctmp = cblistval(doc->nwords, i, &wsiz);
+    cblistpush(tlist, ctmp, wsiz);
   }
+  tmp = cblistdump(tlist, &tsiz);
+  cbmapputvbuf(map, OD_NWORDSEXPR, sizeof(OD_NWORDSEXPR), tmp, tsiz);
+  cblistclose(tlist);
+  tlist = cblistopen();
+  for(i = 0; i < wmax; i++){
+    ctmp = cblistval(doc->awords, i, &wsiz);
+    if(strcmp(ctmp, cblistval(doc->nwords, i, NULL))){
+      cblistpush(tlist, ctmp, wsiz);
+    } else {
+      cblistpush(tlist, "\0", 1);
+    }
+  }
+  tmp = cblistdump(tlist, &tsiz);
+  cbmapputvbuf(map, OD_AWORDSEXPR, sizeof(OD_AWORDSEXPR), tmp, tsiz);
+  cblistclose(tlist);
   tmp = cbmapdump(map, &tsiz);
   cbmapclose(map);
   if(_qdbm_deflate){
@@ -222,15 +251,23 @@ int odput(ODEUM *odeum, ODDOC *doc, int wmax, int over){
   while((word = cbmapiternext(map, &wsiz)) != NULL){
     pair.id = docid;
     pair.score = (int)(*(int *)cbmapget(map, word, wsiz, NULL) / ival);
-    if(!crput(odeum->indexdb, word, wsiz, (char *)&pair, sizeof(pair), CR_DCAT)){
-      cbmapclose(map);
-      odeum->fatal = TRUE;
-      return FALSE;
-    }
+    cbmapputcat(odeum->cachemap, word, wsiz, (char *)&pair, sizeof(pair));
+    cbmapmove(odeum->cachemap, word, wsiz, FALSE);
+    odeum->cacheasiz += sizeof(pair);
     cbmapput(odeum->sortmap, word, wsiz, "", 0, FALSE);
   }
   cbmapclose(map);
+  if(odeum->cacheasiz > odcachesiz){
+    for(i = OD_CFBEGSIZ; odeum->cacheasiz > odcachesiz * OD_CFLIVERAT && i >= OD_CFENDSIZ;
+        i /= 2){
+      if(!odcacheflushfreq(odeum, "odput", i)) return FALSE;
+    }
+    while(odeum->cacheasiz > odcachesiz * OD_CFLIVERAT){
+      if(!odcacheflushrare(odeum, "odput", OD_CFRFRAT)) return FALSE;
+    }
+  }
   doc->id = docid;
+  odeum->ldid = docid;
   return TRUE;
 }
 
@@ -341,8 +378,8 @@ ODDOC *odget(ODEUM *odeum, const char *uri){
 /* Retrieve a document by an ID number. */
 ODDOC *odgetbyid(ODEUM *odeum, int id){
   char *tmp, *zbuf;
-  const char *uritmp, *attrstmp, *nwordstmp, *awordstmp;
-  int tsiz, uritsiz, attrstsiz, nwordstsiz, awordstsiz, zsiz;
+  const char *uritmp, *attrstmp, *nwordstmp, *awordstmp, *asis, *normal;
+  int i, tsiz, uritsiz, attrstsiz, nwordstsiz, awordstsiz, zsiz, asiz, nsiz;
   ODDOC *doc;
   CBMAP *map;
   assert(odeum);
@@ -388,6 +425,13 @@ ODDOC *odgetbyid(ODEUM *odeum, int id){
   doc->nwords = cblistload(nwordstmp, nwordstsiz);
   doc->awords = cblistload(awordstmp, awordstsiz);
   cbmapclose(map);
+  for(i = 0; i < cblistnum(doc->awords); i++){
+    asis = cblistval(doc->awords, i, &asiz);
+    if(asiz == 1 && asis[0] == '\0'){
+      normal = cblistval(doc->nwords, i, &nsiz);
+      cblistover(doc->awords, i, normal, nsiz);
+    }
+  }
   return doc;
 }
 
@@ -441,7 +485,8 @@ ODPAIR *odsearch(ODEUM *odeum, const char *word, int max, int *np){
     dpecode = DP_EFATAL;
     return NULL;
   }
-  if(odeum->wmode && cbmaprnum(odeum->sortmap) > 0 && !odsortindex(odeum)){
+  if(odeum->wmode && cbmaprnum(odeum->sortmap) > 0 &&
+     (!odcacheflush(odeum, "odsearch") || !odsortindex(odeum, "odsearch"))){
     odeum->fatal = TRUE;
     return NULL;
   }
@@ -529,6 +574,7 @@ int odsync(ODEUM *odeum){
     dpecode = DP_EMODE;
     return FALSE;
   }
+  if(odotcb) odotcb("odsync", odeum, "writing meta information");
   sprintf(numbuf, "%d", odeum->dmax);
   if(!vlput(odeum->rdocsdb, OD_DMAXEXPR, sizeof(OD_DMAXEXPR), numbuf, -1, VL_DOVER)){
     odeum->fatal = TRUE;
@@ -539,18 +585,25 @@ int odsync(ODEUM *odeum){
     odeum->fatal = TRUE;
     return FALSE;
   }
-  if(!odsortindex(odeum)){
+  if(!odcacheflush(odeum, "odsync")){
     odeum->fatal = TRUE;
     return FALSE;
   }
+  if(!odsortindex(odeum, "odsync")){
+    odeum->fatal = TRUE;
+    return FALSE;
+  }
+  if(odotcb) odotcb("odsync", odeum, "synchronizing the document database");
   if(!crsync(odeum->docsdb)){
     odeum->fatal = TRUE;
     return FALSE;
   }
+  if(odotcb) odotcb("odsync", odeum, "synchronizing the inverted index");
   if(!crsync(odeum->indexdb)){
     odeum->fatal = TRUE;
     return FALSE;
   }
+  if(odotcb) odotcb("odsync", odeum, "synchronizing the reverse dictionary");
   if(!vlsync(odeum->rdocsdb)){
     odeum->fatal = TRUE;
     return FALSE;
@@ -570,15 +623,21 @@ int odoptimize(ODEUM *odeum){
     dpecode = DP_EMODE;
     return FALSE;
   }
-  if(odotcb) odotcb("odoptimize", odeum, "purging dispensable regions");
-  if(!odpurgeindex(odeum)){
+  if(!odcacheflush(odeum, "odoptimize")){
     odeum->fatal = TRUE;
     return FALSE;
   }
-  if(odotcb) odotcb("odoptimize", odeum, "sorting the inverted index");
-  if(!odsortindex(odeum)){
-    odeum->fatal = TRUE;
-    return FALSE;
+  if(odeum->ldid < 1 || odeum->ldid != odeum->dnum){
+    if(!odpurgeindex(odeum, "odoptimize")){
+      odeum->fatal = TRUE;
+      return FALSE;
+    }
+  }
+  if(odeum->ldid > 0){
+    if(!odsortindex(odeum, "odoptimize")){
+      odeum->fatal = TRUE;
+      return FALSE;
+    }
   }
   if(odotcb) odotcb("odoptimize", odeum, "optimizing the document database");
   if(!croptimize(odeum->docsdb, -1)){
@@ -694,6 +753,13 @@ int odinode(ODEUM *odeum){
 }
 
 
+/* Get the last modified time of a database. */
+int odmtime(ODEUM *odeum){
+  assert(odeum);
+  return crmtime(odeum->indexdb);
+}
+
+
 /* Merge plural database directories. */
 int odmerge(const char *name, const CBLIST *elemnames){
   ODEUM *odeum, **elems;
@@ -721,8 +787,8 @@ int odmerge(const char *name, const CBLIST *elemnames){
     wnum += odwnum(elems[i]);
   }
   dbnum = (int)(dnum * OD_MDBRATIO / OD_DOCSDNUM);
-  ibnum = (int)(((wnum * OD_MIBRATIO) / num) / OD_INDEXDNUM);
-  if(!(odeum = odopendb(name, OD_OWRITER | OD_OCREAT | OD_OTRUNC, dbnum, ibnum))){
+  ibnum = (int)(wnum * OD_MIBRATIO / odindexdnum);
+  if(!(odeum = odopendb(name, OD_OWRITER | OD_OCREAT | OD_OTRUNC, dbnum, ibnum, "odmerge"))){
     for(i = 0; i < num; i++){
       odclose(elems[i]);
     }
@@ -796,16 +862,24 @@ int odmerge(const char *name, const CBLIST *elemnames){
     }
   }
   if(odotcb) odotcb("odmerge", odeum, "sorting the inverted index");
+  tnum = 0;
   if(!criterinit(curia) && dpecode != DP_ENOITEM) err = TRUE;
   while((word = criternext(curia, &wsiz)) != NULL){
     if((vbuf = crget(curia, word, wsiz, 0, -1, &vsiz)) != NULL){
-      pairs = (ODPAIR *)vbuf;
-      pnum = vsiz / sizeof(ODPAIR);
-      cbqsort(pairs, pnum, sizeof(ODPAIR), odsortcompare);
-      if(!crput(curia, word, wsiz, vbuf, vsiz, CR_DOVER)) err = TRUE;
+      if(vsiz > sizeof(ODPAIR)){
+        pairs = (ODPAIR *)vbuf;
+        pnum = vsiz / sizeof(ODPAIR);
+        cbqsort(pairs, pnum, sizeof(ODPAIR), odsortcompare);
+        if(!crput(curia, word, wsiz, vbuf, vsiz, CR_DOVER)) err = TRUE;
+      }
       free(vbuf);
     }
     free(word);
+    if(odotcb && (tnum + 1) % OD_OTPERWORDS == 0){
+      sprintf(otmsg, "... (%d/%d)", tnum + 1, crrnum(curia));
+      odotcb("odmerge", odeum, otmsg);
+    }
+    tnum++;
   }
   if(odotcb) odotcb("odmerge", odeum, "synchronizing the inverted index");
   if(!crsync(curia)) err = TRUE;
@@ -858,7 +932,7 @@ int odmerge(const char *name, const CBLIST *elemnames){
       }
       free(vbuf);
       free(kbuf);
-      odeum->dnum += 1;
+      odeum->dnum++;
       if(odotcb && (tnum + 1) % OD_OTPERDOCS == 0){
         sprintf(otmsg, "... (%d/%d)", tnum + 1, crrnum(ecuria));
         odotcb("odmerge", odeum, otmsg);
@@ -929,7 +1003,7 @@ ODDOC *oddocopen(const char *uri){
   doc = cbmalloc(sizeof(ODDOC));
   doc->id = -1;
   doc->uri = cbmemdup(uri, -1);
-  doc->attrs = cbmapopen();
+  doc->attrs = cbmapopenex(OD_MAPPBNUM);
   doc->nwords = cblistopen();
   doc->awords = cblistopen();
   return doc;
@@ -1039,7 +1113,7 @@ CBMAP *oddocscores(const ODDOC *doc, int max, ODEUM *odeum){
     cbqsort(owords, mnum, sizeof(ODWORD), odwordcompare);
   }
   if(mnum > max) mnum = max;
-  kwmap = cbmapopen();
+  kwmap = cbmapopenex(OD_MAPPBNUM);
   for(i = 0; i < mnum; i++){
     nbsiz = sprintf(numbuf, "%d", owords[i].num);
     cbmapput(kwmap, owords[i].word, -1, numbuf, nbsiz, TRUE);
@@ -1220,9 +1294,158 @@ double odvectorcosine(const int *avec, const int *bvec, int vnum){
 }
 
 
+/* Set the global tuning parameters. */
+void odsettuning(int ibnum, int idnum, int cbnum, int csiz){
+  if(ibnum > 0) odindexbnum = ibnum;
+  if(idnum > 0) odindexdnum = idnum;
+  if(cbnum > 0) odcachebnum = dpprimenum(cbnum);
+  if(csiz > 0) odcachesiz = csiz;
+}
+
+
+/* Break a text into words and store appearance forms and normalized form into lists. */
+void odanalyzetext(ODEUM *odeum, const char *text, CBLIST *awords, CBLIST *nwords){
+  char aword[OD_MAXWORDLEN+1], *wp;
+  int lev, wsiz;
+  assert(odeum && text && awords);
+  lev = OD_EVSPACE;
+  wsiz = 0;
+  for(; *text != '\0'; text++){
+    switch(odeum->statechars[*(unsigned char *)text]){
+    case OD_EVWORD:
+      if(wsiz > 0 && lev == OD_EVDELIM){
+        cblistpush(awords, aword, wsiz);
+        if(nwords) cblistpush(nwords, "", 0);
+        wsiz = 0;
+      }
+      if(wsiz <= OD_MAXWORDLEN){
+        aword[wsiz++] = *text;
+      }
+      lev = OD_EVWORD;
+      break;
+    case OD_EVGLUE:
+      if(wsiz > 0 && lev == OD_EVDELIM){
+        cblistpush(awords, aword, wsiz);
+        if(nwords) cblistpush(nwords, "", 0);
+        wsiz = 0;
+      }
+      if(wsiz <= OD_MAXWORDLEN){
+        aword[wsiz++] = *text;
+      }
+      lev = OD_EVGLUE;
+      break;
+    case OD_EVDELIM:
+      if(wsiz > 0 && lev != OD_EVDELIM){
+        cblistpush(awords, aword, wsiz);
+        if(nwords){
+          wp = aword;
+          aword[wsiz] = '\0';
+          while(*wp != '\0'){
+            if(*wp >= 'A' && *wp <= 'Z') *wp += 'a' - 'A';
+            wp++;
+          }
+          wp--;
+          while(wp >= aword && odeum->statechars[*(unsigned char *)wp] == OD_EVGLUE){
+            wsiz--;
+            wp--;
+          }
+          cblistpush(nwords, aword, wsiz);
+        }
+        wsiz = 0;
+      }
+      if(wsiz <= OD_MAXWORDLEN){
+        aword[wsiz++] = *text;
+      }
+      lev = OD_EVDELIM;
+      break;
+    default:
+      if(wsiz > 0){
+        cblistpush(awords, aword, wsiz);
+        if(nwords){
+          if(lev == OD_EVDELIM){
+            cblistpush(nwords, "", 0);
+          } else {
+            wp = aword;
+            aword[wsiz] = '\0';
+            while(*wp != '\0'){
+              if(*wp >= 'A' && *wp <= 'Z') *wp += 'a' - 'A';
+              wp++;
+            }
+            wp--;
+            while(wp >= aword && odeum->statechars[*(unsigned char *)wp] == OD_EVGLUE){
+              wsiz--;
+              wp--;
+            }
+            cblistpush(nwords, aword, wsiz);
+          }
+        }
+        wsiz = 0;
+      }
+      lev = OD_EVSPACE;
+      break;
+    }
+  }
+  if(wsiz > 0){
+    cblistpush(awords, aword, wsiz);
+    if(nwords){
+      if(lev == OD_EVDELIM){
+        cblistpush(nwords, "", 0);
+      } else {
+        wp = aword;
+        aword[wsiz] = '\0';
+        while(*wp != '\0'){
+          if(*wp >= 'A' && *wp <= 'Z') *wp += 'a' - 'A';
+          wp++;
+        }
+        wp--;
+        while(wp >= aword && odeum->statechars[*(unsigned char *)wp] == OD_EVGLUE){
+          wsiz--;
+          wp--;
+        }
+        cblistpush(nwords, aword, wsiz);
+      }
+    }
+    wsiz = 0;
+  }
+}
+
+
+/* Set the classes of characters used by `odanalyzetext'. */
+void odsetcharclass(ODEUM *odeum, const char *spacechars, const char *delimchars,
+                    const char *gluechars){
+  assert(odeum && spacechars && delimchars && gluechars);
+  memset(odeum->statechars, OD_EVWORD, sizeof(odeum->statechars));
+  for(; *spacechars != '\0'; spacechars++){
+    odeum->statechars[*(unsigned char *)spacechars] = OD_EVSPACE;
+  }
+  for(; *delimchars != '\0'; delimchars++){
+    odeum->statechars[*(unsigned char *)delimchars] = OD_EVDELIM;
+  }
+  for(; *gluechars != '\0'; gluechars++){
+    odeum->statechars[*(unsigned char *)gluechars] = OD_EVGLUE;
+  }
+}
+
+
+/* Query a database using a small boolean query language. */
+ODPAIR *odquery(ODEUM *odeum, const char *query, int *np, CBLIST *errors){
+  CBLIST *tokens = cblistopen();
+  CBLIST *nwords = cblistopen();
+  ODPAIR *results = NULL;
+  assert(odeum && query && np);
+  odanalyzetext(odeum, query, tokens, nwords);
+  odcleannormalized(odeum, nwords);
+  odfixtokens(odeum, tokens);
+  results = odparseexpr(odeum, tokens, nwords, np, errors);
+  cblistclose(tokens);
+  cblistclose(nwords);
+  return results;
+}
+
+
 
 /*************************************************************************************************
- * Features for Experts
+ * features for experts
  *************************************************************************************************/
 
 
@@ -1303,13 +1526,16 @@ double odvecinnerproduct(const int *avec, const int *bvec, int vnum){
    `omode' specifies the connection mode.
    `docsbnum` specifies the number of buckets of the document database.
    `indexbnum` specifies the number of buckets of the index database.
+   `fname' specifies the name of caller function.
    The return value is the database handle or `NULL' if it is not successful. */
-static ODEUM *odopendb(const char *name, int omode, int docsbnum, int indexbnum){
+static ODEUM *odopendb(const char *name, int omode, int docsbnum, int indexbnum,
+                       const char *fname){
   int cromode, vlomode, inode, dmax, dnum;
   char docsname[OD_PATHBUFSIZ], indexname[OD_PATHBUFSIZ], rdocsname[OD_PATHBUFSIZ], *tmp;
   struct stat sbuf;
   CURIA *docsdb, *indexdb;
   VILLA *rdocsdb;
+  CBMAP *cachemap;
   CBMAP *sortmap;
   ODEUM *odeum;
   assert(name);
@@ -1335,6 +1561,10 @@ static ODEUM *odopendb(const char *name, int omode, int docsbnum, int indexbnum)
     cromode |= CR_ONOLCK;
     vlomode |= VL_ONOLCK;
   }
+  if(omode & OD_OLCKNB){
+    cromode |= CR_OLCKNB;
+    vlomode |= VL_OLCKNB;
+  }
   sprintf(docsname, "%s%c%s", name, MYPATHCHR, OD_DOCSNAME);
   sprintf(indexname, "%s%c%s", name, MYPATHCHR, OD_INDEXNAME);
   sprintf(rdocsname, "%s%c%s", name, MYPATHCHR, OD_RDOCSNAME);
@@ -1353,7 +1583,7 @@ static ODEUM *odopendb(const char *name, int omode, int docsbnum, int indexbnum)
   }
   inode = sbuf.st_ino;
   if(!(docsdb = cropen(docsname, cromode, docsbnum, OD_DOCSDNUM))) return NULL;
-  if(!(indexdb = cropen(indexname, cromode, indexbnum, OD_INDEXDNUM))){
+  if(!(indexdb = cropen(indexname, cromode, indexbnum, odindexdnum))){
     crclose(docsdb);
     return NULL;
   }
@@ -1371,8 +1601,10 @@ static ODEUM *odopendb(const char *name, int omode, int docsbnum, int indexbnum)
   }
   vlsettuning(rdocsdb, OD_RDOCSLRM, OD_RDOCSNIM, OD_RDOCSLCN, OD_RDOCSNCN);
   if(omode & OD_OWRITER){
-    sortmap = cbmapopen();
+    cachemap = cbmapopenex(odcachebnum);
+    sortmap = cbmapopenex(odcachebnum);
   } else {
+    cachemap = NULL;
     sortmap = NULL;
   }
   if(vlrnum(rdocsdb) > 0){
@@ -1387,6 +1619,8 @@ static ODEUM *odopendb(const char *name, int omode, int docsbnum, int indexbnum)
       free(tmp);
     }
     if(dmax < 0 || dnum < 0){
+      if(sortmap) cbmapclose(sortmap);
+      if(cachemap) cbmapclose(cachemap);
       vlclose(rdocsdb);
       crclose(indexdb);
       crclose(docsdb);
@@ -1405,38 +1639,155 @@ static ODEUM *odopendb(const char *name, int omode, int docsbnum, int indexbnum)
   odeum->docsdb = docsdb;
   odeum->indexdb = indexdb;
   odeum->rdocsdb = rdocsdb;
+  odeum->cachemap = cachemap;
+  odeum->cacheasiz = 0;
   odeum->sortmap = sortmap;
   odeum->dmax = dmax;
   odeum->dnum = dnum;
+  odeum->ldid = -1;
+  odsetcharclass(odeum, OD_SPACECHARS, OD_DELIMCHARS, OD_GLUECHARS);
+  if(odotcb) odotcb(fname, odeum, "the connection was established");
   return odeum;
 }
 
 
+/* Flush the cache for dirty buffer of words.
+   `odeum' specifies a database handle.
+   `fname' specifies the name of caller function.
+   If successful, the return value is true, else, it is false. */
+static int odcacheflush(ODEUM *odeum, const char *fname){
+  const char *kbuf, *vbuf;
+  char otmsg[OD_OTCBBUFSIZ];
+  int i, rnum, ksiz, vsiz;
+  assert(odeum);
+  if((rnum = cbmaprnum(odeum->cachemap)) < 1) return TRUE;
+  if(odotcb) odotcb(fname, odeum, "flushing caches");
+  cbmapiterinit(odeum->cachemap);
+  for(i = 0; (kbuf = cbmapiternext(odeum->cachemap, &ksiz)) != NULL; i++){
+    vbuf = cbmapget(odeum->cachemap, kbuf, ksiz, &vsiz);
+    if(!crput(odeum->indexdb, kbuf, ksiz, vbuf, vsiz, CR_DCAT)){
+      odeum->fatal = TRUE;
+      return FALSE;
+    }
+    if(odotcb && (i + 1) % OD_OTPERWORDS == 0){
+      sprintf(otmsg, "... (%d/%d)", i + 1, rnum);
+      odotcb(fname, odeum, otmsg);
+    }
+  }
+  cbmapclose(odeum->cachemap);
+  odeum->cachemap = cbmapopenex(odcachebnum);
+  odeum->cacheasiz = 0;
+  return TRUE;
+}
+
+
+/* Flush all frequent words in the cache for dirty buffer of words.
+   `odeum' specifies a database handle.
+   `fname' specifies the name of caller function.
+   `min' specifies the minimum size of frequent words.
+   If successful, the return value is true, else, it is false. */
+static int odcacheflushfreq(ODEUM *odeum, const char *fname, int min){
+  const char *kbuf, *vbuf;
+  char otmsg[OD_OTCBBUFSIZ];
+  int rnum, ksiz, vsiz;
+  assert(odeum);
+  if((rnum = cbmaprnum(odeum->cachemap)) < 1) return TRUE;
+  if(odotcb){
+    sprintf(otmsg, "flushing frequent words: min=%d asiz=%d rnum=%d)",
+            min, odeum->cacheasiz, rnum);
+    odotcb(fname, odeum, otmsg);
+  }
+  cbmapiterinit(odeum->cachemap);
+  while((kbuf = cbmapiternext(odeum->cachemap, &ksiz)) != NULL){
+    vbuf = cbmapget(odeum->cachemap, kbuf, ksiz, &vsiz);
+    if(vsiz >= sizeof(ODPAIR) * min){
+      if(!crput(odeum->indexdb, kbuf, ksiz, vbuf, vsiz, CR_DCAT)){
+        odeum->fatal = TRUE;
+        return FALSE;
+      }
+      cbmapout(odeum->cachemap, kbuf, ksiz);
+      odeum->cacheasiz -= vsiz;
+    }
+  }
+  if(odotcb){
+    sprintf(otmsg, "... (done): min=%d asiz=%d rnum=%d)",
+            min, odeum->cacheasiz, cbmaprnum(odeum->cachemap));
+    odotcb(fname, odeum, otmsg);
+  }
+  return TRUE;
+}
+
+
+/* Flush the half of rare words in the cache for dirty buffer of words.
+   `odeum' specifies a database handle.
+   `fname' specifies the name of caller function.
+   `ratio' specifies the ratio of rare words.
+   If successful, the return value is true, else, it is false. */
+static int odcacheflushrare(ODEUM *odeum, const char *fname, double ratio){
+  const char *kbuf, *vbuf;
+  char otmsg[OD_OTCBBUFSIZ];
+  int i, rnum, limit, ksiz, vsiz;
+  assert(odeum);
+  if((rnum = cbmaprnum(odeum->cachemap)) < 1) return TRUE;
+  if(odotcb){
+    sprintf(otmsg, "flushing rare words: ratio=%.2f asiz=%d rnum=%d)",
+            ratio, odeum->cacheasiz, rnum);
+    odotcb(fname, odeum, otmsg);
+  }
+  cbmapiterinit(odeum->cachemap);
+  limit = rnum * ratio;
+  for(i = 0; i < limit && (kbuf = cbmapiternext(odeum->cachemap, &ksiz)) != NULL; i++){
+    vbuf = cbmapget(odeum->cachemap, kbuf, ksiz, &vsiz);
+    if(!crput(odeum->indexdb, kbuf, ksiz, vbuf, vsiz, CR_DCAT)){
+      odeum->fatal = TRUE;
+      return FALSE;
+    }
+    cbmapout(odeum->cachemap, kbuf, ksiz);
+    odeum->cacheasiz -= vsiz;
+  }
+  if(odotcb){
+    sprintf(otmsg, "... (done): ratio=%.2f asiz=%d rnum=%d)",
+            ratio, odeum->cacheasiz, cbmaprnum(odeum->cachemap));
+    odotcb(fname, odeum, otmsg);
+  }
+  return TRUE;
+}
+
+
 /* Sort the records of inverted index.
    `odeum' specifies a database handle.
+   `fname' specifies the name of caller function.
    If successful, the return value is true, else, it is false. */
-static int odsortindex(ODEUM *odeum){
+static int odsortindex(ODEUM *odeum, const char *fname){
   const char *word;
-  char *tmp;
-  int wsiz, tsiz;
+  char *tmp, otmsg[OD_OTCBBUFSIZ];
+  int i, rnum, wsiz, tsiz;
   ODPAIR *pairs;
   assert(odeum);
+  if((rnum = cbmaprnum(odeum->sortmap)) < 1) return TRUE;
+  if(odotcb) odotcb(fname, odeum, "sorting the inverted index");
   cbmapiterinit(odeum->sortmap);
-  while((word = cbmapiternext(odeum->sortmap, &wsiz)) != NULL){
+  for(i = 0; (word = cbmapiternext(odeum->sortmap, &wsiz)) != NULL; i++){
     if((tmp = crget(odeum->indexdb, word, wsiz, 0, -1, &tsiz)) != NULL){
-      pairs = (ODPAIR *)tmp;
-      cbqsort(pairs, tsiz / sizeof(ODPAIR), sizeof(ODPAIR), odsortcompare);
-      if(!crput(odeum->indexdb, word, wsiz, tmp, tsiz, CR_DOVER)){
-        free(tmp);
-        return FALSE;
+      if(tsiz > sizeof(ODPAIR)){
+        pairs = (ODPAIR *)tmp;
+        cbqsort(pairs, tsiz / sizeof(ODPAIR), sizeof(ODPAIR), odsortcompare);
+        if(!crput(odeum->indexdb, word, wsiz, tmp, tsiz, CR_DOVER)){
+          free(tmp);
+          return FALSE;
+        }
       }
       free(tmp);
     } else if(dpecode != DP_ENOITEM){
       return FALSE;
     }
+    if(odotcb && (i + 1) % OD_OTPERWORDS == 0){
+      sprintf(otmsg, "... (%d/%d)", i + 1, rnum);
+      odotcb(fname, odeum, otmsg);
+    }
   }
   cbmapclose(odeum->sortmap);
-  odeum->sortmap = cbmapopen();
+  odeum->sortmap = cbmapopenex(odcachebnum);
   return TRUE;
 }
 
@@ -1460,13 +1811,17 @@ static int odsortcompare(const void *a, const void *b){
 
 /* Purge the elements of the deleted documents from the inverted index.
    `odeum' specifies a database handle.
+   `fname' specifies the name of caller function.
    If successful, the return value is true, else, it is false. */
-static int odpurgeindex(ODEUM *odeum){
+static int odpurgeindex(ODEUM *odeum, const char *fname){
   ODPAIR *pairs;
-  char *kbuf, *vbuf;
-  int i, ksiz, vsiz, pnum, wi;
+  char *kbuf, *vbuf, otmsg[OD_OTCBBUFSIZ];
+  int i, rnum, tnum, ksiz, vsiz, pnum, wi;
   assert(odeum);
+  if((rnum = crrnum(odeum->indexdb)) < 1) return TRUE;
+  if(odotcb) odotcb(fname, odeum, "purging dispensable regions");
   if(!criterinit(odeum->indexdb)) return FALSE;
+  tnum = 0;
   while(TRUE){
     if(!(kbuf = criternext(odeum->indexdb, &ksiz))){
       if(dpecode != DP_ENOITEM) return FALSE;
@@ -1500,6 +1855,11 @@ static int odpurgeindex(ODEUM *odeum){
     }
     free(vbuf);
     free(kbuf);
+    if(odotcb && (tnum + 1) % OD_OTPERWORDS == 0){
+      sprintf(otmsg, "... (%d/%d)", tnum + 1, rnum);
+      odotcb(fname, odeum, otmsg);
+    }
+    tnum++;
   }
   return TRUE;
 }
@@ -1539,5 +1899,185 @@ static int odwordcompare(const void *a, const void *b){
 }
 
 
+/* Match an operator without taking it off the token list.
+   `odeum' specifies a database handle.
+   `tokens' specifies a list handle of tokens.
+   The return value is whether the next token is an operator. */
+static int odmatchoperator(ODEUM *odeum, CBLIST *tokens){
+  const char *tk = NULL;
+  int tk_len = 0;
+  tk = cblistval(tokens, 0, &tk_len);
+  if(tk && (tk[0] == '&' || tk[0] == '|' || tk[0] == '!')) return 1;
+  return 0;
+}
+
+
+/* Implements the subexpr part of the grammar.
+   `odeum' specifies a database handle.
+   `tokens' specifies a list handle of tokens.
+   `nwords' specifies a list handle of normalized words.
+   `np' specifies the pointer to a variable to which the number of the elements of the return
+   value is assigned.
+   `errors' specifies a list handle into which error messages are stored.
+   The return value is the pointer to an array of document IDs. */
+static ODPAIR *odparsesubexpr(ODEUM *odeum, CBLIST *tokens, CBLIST *nwords, int *np,
+                              CBLIST *errors){
+  char *tk = NULL;
+  int tk_len = 0;
+  char *nword = NULL;  /* used to do the actual search, should match with tokens */
+  ODPAIR *result = NULL;
+  int result_num = 0;
+  int i;
+  double ival;
+  if((tk = cblistshift(tokens, &tk_len)) != NULL){
+    assert(tk != NULL);
+    if(tk[0] == '('){
+      free(tk);
+      /* recurse into expr */
+      result = odparseexpr(odeum, tokens, nwords, &result_num, errors);
+      /* match right token RPAREN */
+      tk = cblistshift(tokens, &tk_len);
+      /* print an error if either we didn't get anything or we didn't get a ) */
+      if(tk == NULL){
+        if(errors) cblistpush(errors, "Expression ended without closing ')'", -1);
+      } else if(tk[0] != ')'){
+        if(errors) cblistpush(errors, "Un-balanced parenthesis.", -1);
+      }
+    } else if(odeum->statechars[*(unsigned char *)tk] == 0){
+      /* Perform odsearch with the next norm word that isn't an operator. */
+      nword = cblistshift(nwords, NULL);
+      assert(nword != NULL);
+      if((result = odsearch(odeum, nword, -1, &result_num)) != NULL){
+        /* TF-IDF tuning */
+        ival = odlogarithm(result_num);
+        ival = (ival * ival) / 4.0;
+        if(ival < 4.0) ival = 4.0;
+        for(i = 0; i < result_num; i++){
+          result[i].score = (int)(result[i].score / ival);
+        }
+      }
+      free(nword);
+    } else {
+      if(errors) cblistpush(errors, "Invalid sub-expression.  Expected '(' or WORD.", -1);
+      result = cbmalloc(1);
+      result_num = 0;
+    }
+    /* done with the token */
+    free(tk);
+  }
+  *np = result_num;
+  return result;
+}
+
+
+/* Implements the actual recursive decent parser for the mini query language.
+   `odeum' specifies a database handle.
+   `tokens' specifies a list handle of tokens.
+   `nwords' specifies a list handle of normalized words.
+   `np' specifies the pointer to a variable to which the number of the elements of the return
+   value is assigned.
+   `errors' specifies a list handle into which error messages are stored.
+   The return value is the pointer to an array of document IDs.
+   It simply parses an initial subexpr, and then loops over as many (operator subexpr)
+   sequences as it can find.  The odmatchoperator function handles injecting a default &
+   between consecutive words. */
+static ODPAIR *odparseexpr(ODEUM *odeum, CBLIST *tokens, CBLIST *nwords, int *np,
+                           CBLIST *errors){
+  ODPAIR *left = NULL;
+  ODPAIR *right = NULL;
+  ODPAIR *temp = NULL;
+  int left_num = 0;
+  int right_num = 0;
+  int temp_num = 0;
+  char *op = NULL;
+  int op_len = 0;
+  if(!(left = odparsesubexpr(odeum, tokens, nwords, &left_num, errors))) return NULL;
+  /* expr ::= subexpr ( op subexpr )* */
+  while(odmatchoperator(odeum, tokens)){
+    op = cblistshift(tokens, &op_len);
+    if(!(right = odparsesubexpr(odeum, tokens, nwords, &right_num, errors))){
+      free(op);
+      free(left);
+      return NULL;
+    }
+    switch(op[0]){
+    case '&':
+      temp = odpairsand(left, left_num, right, right_num, &temp_num);
+      break;
+    case '|':
+      temp = odpairsor(left, left_num, right, right_num, &temp_num);
+      break;
+    case '!':
+      temp = odpairsnotand(left, left_num, right, right_num, &temp_num);
+      break;
+    default:
+      if(errors) cblistpush(errors, "Invalid operator.  Expected '&', '|', or '!'.", -1);
+      break;
+    }
+    if(temp){
+      /* an operator was done so we must swap it with the left */
+      free(left); left = NULL;
+      left = temp;
+      left_num = temp_num;
+    }
+    free(op);
+    if(right) free(right);
+  }
+  *np = left_num;
+  return left;
+}
+
+
+/* Processes the tokens in order to break them up further.
+   `odeum' specifies a database handle.
+   `tokens' specifies a list handle of tokens. */
+static void odfixtokens(ODEUM *odeum, CBLIST *tokens){
+  const char *tk = NULL;
+  int tk_len = 0;
+  int i = 0;
+  int lastword = 0;
+  for(i = 0; i < cblistnum(tokens); i++){
+    tk = cblistval(tokens, i, &tk_len);
+    assert(tk);
+    if(tk[0] == '&' || tk[0] == '|' || tk[0] == '!' || tk[0] == '(' || tk[0] == ')'){
+      lastword = 0;
+      if(tk_len > 1){
+        /* need to break it up for the next loop around */
+        tk = cblistremove(tokens, i, &tk_len);
+        cblistinsert(tokens, i, tk, 1);
+        cblistinsert(tokens, i+1, tk+1, tk_len-1);
+        free((char *)tk);
+      }
+    } else if(odeum->statechars[*(unsigned char *)tk] == 0){
+      /* if the last one was a word and this is a word then we need a default & between them */
+      if(lastword){
+        cblistinsert(tokens, i, "&", 1);
+        i++;
+      }
+      lastword = 1;
+    }
+  }
+}
+
+
+/* Cleans out the parts of the normalized word list that are not considered words.
+   `odeum' specifies a database handle.
+   `tokens' specifies a list handle of tokens. */
+static void odcleannormalized(ODEUM *odeum, CBLIST *nwords){
+  char *tk = NULL;
+  int tk_len = 0;
+  int i = 0;
+  for(i = 0; i < cblistnum(nwords); i++){
+    tk = (char *)cblistval(nwords, i, &tk_len);
+    if(tk_len == 0 || (!odeum->statechars[*(unsigned char *)tk] == 0)){
+      /* not a word so delete it */
+      tk = cblistremove(nwords, i, &tk_len);
+      free(tk);
+      i--;
+    }
+  }
+}
+
+
 
 /* END OF FILE */
