@@ -28,7 +28,7 @@
 #undef FALSE
 #define FALSE          0                 /* boolean false */
 
-#define DOCBUFSIZ      32                /* buffer for documents */
+#define DOCBUFSIZ      256               /* buffer for documents */
 
 
 /* for RISC OS */
@@ -353,8 +353,10 @@ int docombo(const char *name){
   ODEUM *odeum;
   ODDOC *doc;
   const CBLIST *nwords, *awords;
+  CBLIST *tawords, *tnwords, *oawords;
   ODPAIR *pairs;
-  char buf[DOCBUFSIZ];
+  const char *asis;
+  char buf[DOCBUFSIZ], *normal;
   int i, j, pnum;
   printfflush("<Combination Test>\n  name=%s\n\n", name);
   printfflush("Creating a database with ... ");
@@ -514,6 +516,57 @@ int docombo(const char *name){
     }
   }
   printfflush("ok\n");
+  printfflush("Analyzing text ... ");
+  tawords = cblistopen();
+  tnwords = cblistopen();
+  odanalyzetext(odeum, "I'd like to ++see++ Mr. X-men tomorrow.", tawords, tnwords);
+  odanalyzetext(odeum, "=== :-) SMILE . @ . SAD :-< ===", tawords, tnwords);
+  for(i = 0; i < DOCBUFSIZ - 1; i++){
+    buf[i] = myrand() % 255 + 1;
+  }
+  buf[DOCBUFSIZ-1] = '\0';
+  cblistclose(tnwords);
+  cblistclose(tawords);
+  for(i = 0; i < 1000; i++){
+    for(j = 0; j < DOCBUFSIZ - 1; j++){
+      if((j + 1) % 32 == 0){
+        buf[j] = ' ';
+      } else {
+        buf[j] = myrand() % 255 + 1;
+      }
+    }
+    buf[DOCBUFSIZ-1] = '\0';
+    tawords = cblistopen();
+    tnwords = cblistopen();
+    odanalyzetext(odeum, buf, tawords, tnwords);
+    oawords = odbreaktext(buf);
+    if(cblistnum(tawords) != cblistnum(oawords) || cblistnum(tnwords) != cblistnum(oawords)){
+      fprintf(stderr, "%s: %s: invalid analyzing\n", progname, name);
+      cblistclose(oawords);
+      cblistclose(tnwords);
+      cblistclose(tawords);
+      odclose(odeum);
+      return 1;
+    }
+    for(j = 0; j < cblistnum(oawords); j++){
+      asis = cblistval(oawords, j, NULL);
+      normal = odnormalizeword(asis);
+      if(strcmp(asis, cblistval(oawords, j, NULL)) || strcmp(normal, cblistval(tnwords, j, NULL))){
+        fprintf(stderr, "%s: %s: invalid analyzing\n", progname, name);
+        free(normal);
+        cblistclose(oawords);
+        cblistclose(tnwords);
+        cblistclose(tawords);
+        odclose(odeum);
+        return 1;
+      }
+      free(normal);
+    }
+    cblistclose(oawords);
+    cblistclose(tnwords);
+    cblistclose(tawords);
+  }
+  printfflush("ok\n");
   printfflush("Closing the database ... ");
   if(!odclose(odeum)){
     pdperror(name);
