@@ -34,7 +34,8 @@ use constant OWRITER => 1 << 1;          # open as a writer
 use constant OCREAT => 1 << 2;           # a writer creating
 use constant OTRUNC => 1 << 3;           # a writer truncating
 use constant ONOLCK => 1 << 4;           # open without locking
-use constant OSPARSE => 1 << 5;          # create as sparse files
+use constant OLCKNB => 1 << 5;           # lock without blocking
+use constant OSPARSE => 1 << 6;          # create as sparse files
 
 use constant DOVER => 0;                 # overwrite an existing value
 use constant DKEEP => 1;                 # keep an existing value
@@ -72,9 +73,9 @@ our($errmsg) = "no error";               # message of the last error
 # `Curia::OCREAT', which means it creates a new database if not exist, `Curia::OTRUNC', which
 # means it creates a new database regardless if one exists.  Both of `Curia::OREADER' and
 # `Curia::OWRITER' can be added to by bitwise or: `Curia::ONOLCK', which means it opens a
-# database directory without file locking.  `Curia::OCREAT' can be added to by bitwise or:
-# `Curia::OSPARSE', which means it creates database files as sparse files.  If it is undef,
-# `Curia::OREADER' is specified.
+# database directory without file locking, or `Curia::OLCKNB', which means locking is performed
+# without blocking.  `Curia::OCREAT' can be added to by bitwise or: `Curia::OSPARSE', which means
+# it creates database files as sparse files.  If it is undef, `Curia::OREADER' is specified.
 # `$bnum' specifies the number of elements of the bucket array.  If it is undef or not more
 # than 0, the default value is specified.  The size of a bucket array is determined on creating,
 # and can not be changed except for by optimization of the database.  Suggested size of a
