@@ -34,7 +34,8 @@ use constant OWRITER => 1 << 1;          # open as a writer
 use constant OCREAT => 1 << 2;           # a writer creating
 use constant OTRUNC => 1 << 3;           # a writer truncating
 use constant ONOLCK => 1 << 4;           # open without locking
-use constant OSPARSE => 1 << 5;          # create as a sparse file
+use constant OLCKNB => 1 << 5;           # lock without blocking
+use constant OSPARSE => 1 << 6;          # create as a sparse file
 
 use constant DOVER => 0;                 # overwrite an existing value
 use constant DKEEP => 1;                 # keep an existing value
@@ -72,9 +73,9 @@ our($errmsg) = "no error";               # message of the last error
 # `Depot::OCREAT', which means it creates a new database if not exist, `Depot::OTRUNC', which
 # means it creates a new database regardless if one exists.  Both of `Depot::OREADER' and
 # `Depot::OWRITER' can be added to by bitwise or: `Depot::ONOLCK', which means it opens a
-# database file without file locking.  `Depot::OCREAT' can be added to by bitwise or:
-# `Depot::OSPARSE', which means it creates a database file as a sparse file.  If it is undef,
-# `Depot::OREADER' is specified.
+# database file without file locking, or `Depot::OLCKNB', which means locking is performed
+# without blocking.  `Depot::OCREAT' can be added to by bitwise or: `Depot::OSPARSE', which means
+# it creates a database file as a sparse file.  If it is undef, `Depot::OREADER' is specified.
 # `$bnum' specifies the number of elements of the bucket array.  If it is undef or not more
 # than 0, the default value is specified.  The size of a bucket array is determined on creating,
 # and can not be changed except for by optimization of the database.  Suggested size of a
