@@ -34,7 +34,8 @@ use constant OWRITER => 1 << 1;          # open as a writer
 use constant OCREAT => 1 << 2;           # a writer creating
 use constant OTRUNC => 1 << 3;           # a writer truncating
 use constant ONOLCK => 1 << 4;           # open without locking
-use constant OZCOMP => 1 << 5;           # compress leaves
+use constant OLCKNB => 1 << 5;           # lock without blocking
+use constant OZCOMP => 1 << 6;           # compress leaves
 
 use constant CMPLEX => 0;                # compare in lexical order
 use constant CMPDEC => 1;                # compare decimal strings
@@ -78,8 +79,9 @@ our($errmsg) = "no error";               # message of the last error
 # `Villa::OCREAT', which means it creates a new database if not exist, `Villa::OTRUNC', which
 # means it creates a new database regardless if one exists, `Villa::OZCOMP', which means leaves
 # in the database are compressed.  Both of `Villa::OREADER' and `Villa::OWRITER' can be added to
-# by bitwise or: `Villa::ONOLCK', which means it opens a database file without file locking.  If
-# it is undef, `Villa::OREADER' is specified.
+# by bitwise or: `Villa::ONOLCK', which means it opens a database file without file locking, or
+# `Villa::OLCKNB', which means locking is performed without blocking.  If it is undef,
+# `Villa::OREADER' is specified.
 # `cmode' specifies the comparing function: `Villa::CMPLEX' comparing keys in lexical order,
 # `Villa::CMPDEC' comparing keys as decimal strings.  The comparing function should be kept
 # same in the life of a database.
