@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * Implementation of Villa
- *                                                      Copyright (C) 2000-2004 Mikio Hirabayashi
+ *                                                      Copyright (C) 2000-2005 Mikio Hirabayashi
  * This file is part of QDBM, Quick Database Manager.
  * QDBM is free software; you can redistribute it and/or modify it under the terms of the GNU
  * Lesser General Public License as published by the Free Software Foundation; either version
@@ -20,6 +20,7 @@
 #define VL_LEAFIDMIN   1                 /* minimum number of leaf ID */
 #define VL_NODEIDMIN   100000000         /* minimum number of node ID */
 #define VL_VNUMBUFSIZ  8                 /* size of a buffer for variable length number */
+#define VL_NUMBUFSIZ   32                /* size of a buffer for a number */
 #define VL_LEVELMAX    64                /* max level of B+ tree */
 #define VL_DEFLRECMAX  49                /* default number of records in each leaf */
 #define VL_DEFNIDXMAX  192               /* default number of indexes in each node */
@@ -98,6 +99,7 @@ VILLA *vlopen(const char *name, int omode, VLCFUNC cmp){
     if(omode & VL_OTRUNC) dpomode |= DP_OTRUNC;
   }
   if(omode & VL_ONOLCK) dpomode |= DP_ONOLCK;
+  if(omode & VL_OLCKNB) dpomode |= DP_OLCKNB;
   if(!(depot = dpopen(name, dpomode, VL_INITBNUM))) return NULL;
   flags = dpgetflags(depot);
   zmode = 0;
@@ -128,7 +130,7 @@ VILLA *vlopen(const char *name, int omode, VLCFUNC cmp){
       return NULL;
     }
   }
-  villa = cbmalloc(sizeof(VILLA));
+  CB_MALLOC(villa, sizeof(VILLA));
   villa->depot = depot;
   villa->cmp = cmp;
   villa->wmode = (omode & VL_OWRITER);
@@ -159,6 +161,10 @@ VILLA *vlopen(const char *name, int omode, VLCFUNC cmp){
     leaf = vlleafnew(villa, -1, -1);
     villa->root = leaf->id;
     villa->last = leaf->id;
+    if(!vltranbegin(villa) || !vltranabort(villa)){
+      vlclose(villa);
+      return NULL;
+    }
   }
   return villa;
 }
@@ -350,7 +356,7 @@ int vlvnum(VILLA *villa, const char *kbuf, int ksiz){
 
 
 /* Store plural records corresponding a key. */
-int vlputlist(VILLA *villa, const char *kbuf, int ksiz, CBLIST *vals){
+int vlputlist(VILLA *villa, const char *kbuf, int ksiz, const CBLIST *vals){
   int i, vsiz;
   const char *vbuf;
   assert(villa && kbuf && vals);
@@ -364,7 +370,7 @@ int vlputlist(VILLA *villa, const char *kbuf, int ksiz, CBLIST *vals){
   }
   if(ksiz < 0) ksiz = strlen(kbuf);
   for(i = 0; i < CB_LISTNUM(vals); i++){
-    vbuf = cblistval(vals, i, &vsiz);
+    vbuf = CB_LISTVAL2(vals, i, &vsiz);
     if(!vlput(villa, kbuf, ksiz, vbuf, vsiz, VL_DDUP)) return FALSE;
   }
   return TRUE;
@@ -407,7 +413,7 @@ CBLIST *vlgetlist(VILLA *villa, const char *kbuf, int ksiz){
   cblistpush(vals, CB_DATUMPTR(recp->first), CB_DATUMSIZE(recp->first));
   if(recp->rest){
     for(i = 0; i < CB_LISTNUM(recp->rest); i++){
-      vbuf = cblistval(recp->rest, i, &vsiz);
+      vbuf = CB_LISTVAL2(recp->rest, i, &vsiz);
       cblistpush(vals, vbuf, vsiz);
     }
   }
@@ -419,6 +425,42 @@ CBLIST *vlgetlist(VILLA *villa, const char *kbuf, int ksiz){
 }
 
 
+/* Retrieve concatenated values of all records corresponding a key. */
+char *vlgetcat(VILLA *villa, const char *kbuf, int ksiz, int *sp){
+  VLLEAF *leaf;
+  VLREC *recp;
+  int pid, i, vsiz, rsiz;
+  char *rbuf;
+  const char *vbuf;
+  assert(villa && kbuf);
+  if(ksiz < 0) ksiz = strlen(kbuf);
+  if((pid = vlsearchleaf(villa, kbuf, ksiz, NULL, NULL)) == -1) return NULL;
+  if(!(leaf = vlleafload(villa, pid))) return NULL;
+  if(!(recp = vlrecsearch(villa, leaf, kbuf, ksiz, NULL))){
+    dpecodeset(DP_ENOITEM, __FILE__, __LINE__);
+    return NULL;
+  }
+  rsiz = CB_DATUMSIZE(recp->first);
+  CB_MALLOC(rbuf, rsiz + 1);
+  memcpy(rbuf, CB_DATUMPTR(recp->first), rsiz);
+  if(recp->rest){
+    for(i = 0; i < CB_LISTNUM(recp->rest); i++){
+      vbuf = CB_LISTVAL2(recp->rest, i, &vsiz);
+      CB_REALLOC(rbuf, rsiz + vsiz + 1);
+      memcpy(rbuf + rsiz, vbuf, vsiz);
+      rsiz += vsiz;
+    }
+  }
+  rbuf[rsiz] = '\0';
+  if(!villa->tran && !vlcacheadjust(villa)){
+    free(rbuf);
+    return NULL;
+  }
+  if(sp) *sp = rsiz;
+  return rbuf;
+}
+
+
 /* Move the cursor to the first record. */
 int vlcurfirst(VILLA *villa){
   VLLEAF *leaf;
@@ -680,7 +722,7 @@ char *vlcurval(VILLA *villa, int *sp){
     kbuf = CB_DATUMPTR(recp->first);
     ksiz = CB_DATUMSIZE(recp->first);
   } else {
-    kbuf = cblistval(recp->rest, villa->curvnum - 1, &ksiz);
+    kbuf = CB_LISTVAL2(recp->rest, villa->curvnum - 1, &ksiz);
   }
   if(sp) *sp = ksiz;
   return cbmemdup(kbuf, ksiz);
@@ -815,6 +857,13 @@ int vlinode(VILLA *villa){
 }
 
 
+/* Get the last modified time of a database. */
+int vlmtime(VILLA *villa){
+  assert(villa);
+  return dpmtime(villa->depot);
+}
+
+
 /* Begin the transaction. */
 int vltranbegin(VILLA *villa){
   int err, pid;
@@ -1066,6 +1115,79 @@ int vlrepair(const char *name, VLCFUNC cmp){
 }
 
 
+/* Dump all records as endian independent data. */
+int vlexportdb(VILLA *villa, const char *name){
+  DEPOT *depot;
+  char path[VL_PATHBUFSIZ], *kbuf, *vbuf, *nkey;
+  int i, err, ksiz, vsiz, ki;
+  assert(villa && name);
+  sprintf(path, "%s%s", name, VL_TMPFSUF);
+  if(!(depot = dpopen(path, DP_OWRITER | DP_OCREAT | DP_OTRUNC, -1))) return FALSE;
+  err = FALSE;
+  vlcurfirst(villa);
+  for(i = 0; !err && (kbuf = vlcurkey(villa, &ksiz)) != NULL; i++){
+    if((vbuf = vlcurval(villa, &vsiz)) != NULL){
+      CB_MALLOC(nkey, ksiz + VL_NUMBUFSIZ);
+      ki = sprintf(nkey, "%X\t", i);
+      memcpy(nkey + ki, kbuf, ksiz);
+      if(!dpput(depot, nkey, ki + ksiz, vbuf, vsiz, DP_DKEEP)) err = TRUE;
+      free(nkey);
+      free(vbuf);
+    } else {
+      err = TRUE;
+    }
+    free(kbuf);
+    vlcurnext(villa);
+  }
+  if(!dpexportdb(depot, name)) err = TRUE;
+  if(!dpclose(depot)) err = TRUE;
+  if(!dpremove(path)) err = TRUE;
+  return !err && !vlfatalerror(villa);
+}
+
+
+/* Load all records from endian independent data. */
+int vlimportdb(VILLA *villa, const char *name){
+  DEPOT *depot;
+  char path[VL_PATHBUFSIZ], *kbuf, *vbuf, *rp;
+  int err, ksiz, vsiz;
+  assert(villa && name);
+  if(!villa->wmode){
+    dpecodeset(DP_EMODE, __FILE__, __LINE__);
+    return FALSE;
+  }
+  if(vlrnum(villa) > 0){
+    dpecodeset(DP_EMISC, __FILE__, __LINE__);
+    return FALSE;
+  }
+  kbuf = dpname(villa->depot);
+  sprintf(path, "%s%s", kbuf, VL_TMPFSUF);
+  free(kbuf);
+  if(!(depot = dpopen(path, DP_OWRITER | DP_OCREAT | DP_OTRUNC, -1))) return FALSE;
+  err = FALSE;
+  if(!dpimportdb(depot, name)) err = TRUE;
+  dpiterinit(depot);
+  while(!err && (kbuf = dpiternext(depot, &ksiz)) != NULL){
+    if((vbuf = dpget(depot, kbuf, ksiz, 0, -1, &vsiz)) != NULL){
+      if((rp = strchr(kbuf, '\t')) != NULL){
+        rp++;
+        if(!vlput(villa, rp, ksiz - (rp - kbuf), vbuf, vsiz, VL_DDUP)) err = TRUE;
+      } else {
+        dpecodeset(DP_EBROKEN, __FILE__, __LINE__);
+        err = TRUE;
+      }
+      free(vbuf);
+    } else {
+      err = TRUE;
+    }
+    free(kbuf);
+  }
+  if(!dpclose(depot)) err = TRUE;
+  if(!dpremove(path)) err = TRUE;
+  return !err && !vlfatalerror(villa);
+}
+
+
 
 /*************************************************************************************************
  * private objects
@@ -1309,7 +1431,7 @@ static int vlleafsave(VILLA *villa, VLLEAF *leaf){
     cbdatumcat(buf, CB_DATUMPTR(recp->first), vsiz);
     if(recp->rest){
       for(j = 0; j < CB_LISTNUM(recp->rest); j++){
-        vbuf = cblistval(recp->rest, j, &vsiz);
+        vbuf = CB_LISTVAL2(recp->rest, j, &vsiz);
         vnumsiz = vlsetvnumbuf(vnumbuf, vsiz);
         cbdatumcat(buf, vnumbuf, vnumsiz);
         cbdatumcat(buf, vbuf, vsiz);
@@ -1363,7 +1485,10 @@ static VLLEAF *vlleafload(VILLA *villa, int id){
   ksiz = -1;
   prev = -1;
   next = -1;
-  if(!(buf = dpget(villa->depot, (char *)&id, sizeof(int), 0, -1, &size))) return NULL;
+  if(!(buf = dpget(villa->depot, (char *)&id, sizeof(int), 0, -1, &size))){
+    dpecodeset(DP_EBROKEN, __FILE__, __LINE__);
+    return NULL;
+  }
   if(_qdbm_inflate && villa->zmode){
     if(!(zbuf = _qdbm_inflate(buf, size, &zsiz))){
       dpecodeset(DP_EBROKEN, __FILE__, __LINE__);
